import os
from collections import defaultdict

import pandas as pd
import scipy

from nanocompare.global_config import pic_base_dir
from nanocompare.global_settings import ToolNameList

perfInfileName = os.path.join('/projects/li-lab/yang/results/2021-08-07/modify_cpg_bed',
                              'performance-results-METEORE.csv')
distributionInfileName = os.path.join('/projects/li-lab/yang/results/2021-08-07/modify_cpg_bed',
                                      'all.certain.sites.distribution.each.genomic.region.METEORE.cov5.csv')

genomic_regions = \
    ["Promoters", "Exons", "Introns", "Intergenic", "CpG Island", "CpG Shores", "CpG Shelves"] + \
    ['CG_20', 'CG_40', 'CG_60', 'CG_80', 'CG_100'] + \
    ['rep_SINE', 'rep_LINE', 'rep_LTR', 'rep_DNA', 'rep_Others']

if False:
    perfInfileName = os.path.join('/projects/li-lab/yang/results/2021-08-15', 'performance-results-seven.csv')
    distributionInfileName = os.path.join('/projects/li-lab/yang/results/2021-08-15',
                                          'all.certain.sites.distribution.each.genomic.region.seven.cov5.table.s6.csv')
    genomic_regions = \
        ["Promoters", "Exons", "Introns", "Intergenic", "CpG islands", "CpG shores", "CpG shelves"] + \
        ['CG_20', 'CG_40', 'CG_60', 'CG_80', 'CG_100'] + \
        ['rep_SINE', 'rep_LINE', 'rep_LTR', 'rep_DNA', 'rep_Others']

perfDf = pd.read_csv(perfInfileName).loc[:, ['Dataset', 'Tool', 'Location', 'Macro-F1']]
perfDf = perfDf.loc[perfDf['Location'].isin(genomic_regions)]
perfDf = perfDf[perfDf['Dataset'].isin(['NA12878', 'NA19240', 'APL', 'K562'])]
print(len(perfDf))

distDf = pd.read_csv(distributionInfileName).loc[:,
         ['Dataset', 'Coord', 'Singletons', 'Non-singletons', 'Concordant', 'Discordant']]
distDf = distDf.loc[distDf['Coord'].isin(genomic_regions)]
distDf = distDf[distDf['Dataset'].isin(['NA12878', 'NA19240', 'APL', 'K562'])]
distDf['%Non-singletons'] = distDf['Non-singletons'] / (distDf['Non-singletons'] + distDf['Singletons'])
distDf['%Discordant'] = distDf['Discordant'] / (distDf['Discordant'] + distDf['Concordant'])

print(len(distDf))

df = perfDf.merge(distDf, left_on=['Dataset', 'Location'], right_on=['Dataset', 'Coord'])
dataset = defaultdict(list)
for toolName in ToolNameList:
    ndf = df[df['Tool'] == toolName]
    print(len(ndf))
    xx = ndf['%Non-singletons']
    yy = ndf['Macro-F1']
    from scipy import stats

    coe, pvalue = scipy.stats.pearsonr(xx, yy)
    print(f"{toolName}:coe={coe}, pvalue={pvalue}")
    dataset['Method'].append(toolName)
    dataset['COE'].append(coe)
    dataset['P-value'].append(pvalue)
outdf = pd.DataFrame.from_dict(dataset)
outfn = os.path.join(pic_base_dir, 'coe.influence.genomic.regions.percent.nonsingleton.csv')
outdf.to_csv(outfn)

dataset = defaultdict(list)
for toolName in ToolNameList:
    ndf = df[df['Tool'] == toolName]
    print(len(ndf))
    xx = ndf['%Discordant']
    yy = ndf['Macro-F1']

    coe, pvalue = scipy.stats.pearsonr(xx, yy)
    print(f"{toolName}:coe={coe}, pvalue={pvalue}")
    dataset['Method'].append(toolName)
    dataset['COE'].append(coe)
    dataset['P-value'].append(pvalue)
outdf = pd.DataFrame.from_dict(dataset)
outfn = os.path.join(pic_base_dir, 'coe.influence.genomic.regions.percent.discordant.csv')
outdf.to_csv(outfn)



#!/usr/bin/env python

import os, sys, string
import multiprocessing
import time;
import glob
from collections import defaultdict;


def mprint(mstr):
   print(mstr); sys.stdout.flush()

if len(sys.argv)<4:
   print ("Usage: python {} pred_folder-of-DeepMod Base-of-interest unique-fileid-in-sum-file [chr-list]".format(sys.argv[0]))
   print ("       pred_folder-of-DeepMod: the prediction must in its sub-folder.")
   sys.exit(1)

pred_folder = sys.argv[1]
baseofint = sys.argv[2]
sum_fileid = sys.argv[3]


if len(sys.argv)>4:
   chrkeys = ["%s" % cid for cid in sys.argv[4].split(',')]
else:
   chrkeys = []
   for i in range(1, 23):
      chrkeys.append("chr%d" % i)
   chrkeys.append("chrX")
   chrkeys.append("chrY")
   chrkeys.append("chrM")

chrkeys = set(chrkeys)

#####################################
def readbed2(bedf):
   cur_ad = defaultdict();
   with open(bedf, 'r') as mr:
      while True:
         line = mr.readline();
         if not line: break;
         line = line.strip();
         lsp = line.split();
         cur_ad[(lsp[0], int(lsp[1]), lsp[5])] = [int(lsp[9]), int(lsp[11])]  ## Ref: https://github.com/WGLab/DeepMod/blob/master/docs/Results_explanation.md#2-format-of-output
   return cur_ad

def mergeMod(g_ad, cur_ad):
    for k in cur_ad:
       if k in g_ad:
            g_ad[k][0] += cur_ad[k][0]
            g_ad[k][1] += cur_ad[k][1]
       else: g_ad[k] = cur_ad[k]
      
def save_mod(res_file, amod_dict, baseOfin):
   poskeys = sorted(list(amod_dict.keys()))
   for pk in poskeys:
      if amod_dict[pk][1]==0: del amod_dict[pk] ## We correct bug for DeepMod using [0] instead of [1], TODO:

   poskeys = list(amod_dict.keys())
   poskeys = sorted(poskeys);
   with open(res_file, 'w') as mw:
      for pk in poskeys:
          mw.write('%s %d %d %s %d %s  %d %d 0,0,0 %d %d %d\n' % (pk[0], pk[1],pk[1]+1, baseOfin,amod_dict[pk][0] if amod_dict[pk][0]<1000 else 1000,pk[2], pk[1],pk[1]+1, amod_dict[pk][0], int(amod_dict[pk][1]*100/amod_dict[pk][0]) if amod_dict[pk][0]>0 else 0, amod_dict[pk][1]  ))
       

def sum_amod_handler(run_Q):
   handli = 0;
   while not run_Q.empty():
      try:
          ck, pred_folder, baseOfin = run_Q.get(block=False)
          #print (ck, pred_folder, baseOfin); continue
      except:
         break;

      allbedfiles = glob.glob(os.path.join(pred_folder, ("*/*/*/*.%s-.%s.bed" % (ck, baseOfin)) ))
      allbedfiles.extend(glob.glob(os.path.join(pred_folder, ("*/*/*.%s-.%s.bed" % (ck, baseOfin)) )))
      allbedfiles.extend(glob.glob(os.path.join(pred_folder, ("*/*.%s-.%s.bed" % (ck, baseOfin)) )))
      mprint ("%s - %s: %d" % (ck, baseOfin, len(allbedfiles) )) 
      allbedfiles.extend(glob.glob(os.path.join(pred_folder, ("*/*/*/*.%s+.%s.bed" % (ck, baseOfin)) )))
      allbedfiles.extend(glob.glob(os.path.join(pred_folder, ("*/*/*.%s+.%s.bed" % (ck, baseOfin)) )))
      allbedfiles.extend(glob.glob(os.path.join(pred_folder, ("*/*.%s+.%s.bed" % (ck, baseOfin)) )))
      mprint ("%s -+ %s: %d" % (ck, baseOfin, len(allbedfiles) ))

      #  0    1     2     3 4 5   6      7      8   9 0 1
      #chr1 949802 949803 T 1 - 949802 949803 0,0,0 1 0 0 
      amod_dict = defaultdict();      
      res_file = "%s/%s.%s.%s.bed" % (pred_folder, sum_fileid, ck, baseOfin)
      for bedf_ind in range(len(allbedfiles)):
          mprint("\t %s %s %d/%d" % (ck, baseOfin, bedf_ind+1, len(allbedfiles)))
          cur_ad = readbed2(allbedfiles[bedf_ind])
          mergeMod(amod_dict, cur_ad)


      save_mod(res_file, amod_dict, baseOfin)

##############################
pmanager = multiprocessing.Manager();
run_Q = pmanager.Queue();
for ck in chrkeys:
   run_Q.put((ck, pred_folder, baseofint))

mhandlers = [];
share_var = (run_Q, )
m_thread_num = len(chrkeys);
for i in range(m_thread_num+1):
   p = multiprocessing.Process(target=sum_amod_handler, args=share_var)
   p.start();
   mhandlers.append(p);
while any(p.is_alive() for p in mhandlers):
   try:
      time.sleep(1);
   except:
      time.sleep(1);
      continue;



"""
We combine all chr methylation-calling results into a file for each tool
"""
import glob
import gzip
import os
import re
import sys

from nanocompare.global_config import set_log_debug_level, logger
from nanocompare.global_settings import humanChrSet

input_dir = "/projects/li-lab/Nanopore_compare/data/NA12878"
combine_dir = "/projects/li-lab/Nanopore_compare/data/NA12878/combine_allchrs"

## chr 1-22, chrX, chrY
# humanChrSet = [f'chr{k}' for k in range(1, 23)] + ['chrX', 'chrY']

toolSuffix = {
        'Nanopolish'     : 'nanopolish.methylation_calls',
        'Tombo'          : 'tombo.perReadsStats',
        'DeepSignal'     : 'deepsignal.call_mods',
        'DeepMod.C'      : 'deepmod.C',
        'DeepMod.Cluster': 'deepmod.C_clusterCpG',
        'Guppy'          : 'guppy.fast5mod_site_level',
        'Guppy.gcf52ref' : 'guppy.gcf52ref_read_level',
        'Megalodon'      : 'megalodon.per_read',
        }

sep_dict = {'Nanopolish' : '\t', 'Tombo': '\t', 'Megalodon': '\t', 'DeepSignal': '\t', 'DeepMod.C': ' ',
        'DeepMod.Cluster': ' ', 'Guppy': '\t', 'Guppy.gcf52ref': '\t'
        }
chr_col_dict = {'Nanopolish': 0, 'Tombo': 0, 'Megalodon': 1, 'DeepSignal': 0, 'DeepMod.C': 0,
        'DeepMod.Cluster'   : 0, 'Guppy': 0, 'Guppy.gcf52ref': 0}

if __name__ == '__main__':
    set_log_debug_level()

    # toolName = 'Nanopolish'
    toolName = sys.argv[1]

    os.makedirs(combine_dir, exist_ok=True)
    fnlist = glob.glob(os.path.join(input_dir, f'*/NA12878*.{toolSuffix[toolName]}.combine.*.gz'))
    logger.info(f'Find total files={len(fnlist)}')

    processDict = {}
    for fn in fnlist:
        ## such as: NA12878-CHR17.nanopolish.methylation_calls.combine.tsv.gz
        temp_name = os.path.basename(fn)
        matchObj = re.match(rf'NA12878.*(CHR.*)\.{toolSuffix[toolName]}\.combine\.(tsv|bed)\.gz', temp_name)
        if matchObj:
            chr = matchObj.group(1)
            bed_or_tsv = matchObj.group(2)
        else:
            raise Exception(f"regularEx match failed for: {temp_name}")
        processDict[chr] = fn

    logger.info(f'Construct total files={len(processDict)}')
    outfn = os.path.join(combine_dir, f'NA12878.{toolSuffix[toolName]}.combine_allchrs.{bed_or_tsv}.gz')
    with gzip.open(outfn, 'wt') as outf:
        for chr in humanChrSet:
            cntLines = 0
            if chr.upper() not in processDict:
                continue
            fn = processDict[chr.upper()]
            logger.info('\n\n==============================')
            logger.info(f'Start output: {fn}')
            with gzip.open(fn, 'rt')  as inf:
                for row in inf:
                    tmp = row.strip().split(sep_dict[toolName])
                    if tmp[chr_col_dict[toolName]] != chr:  ## filter out other chr outputs
                        continue
                    cntLines += 1
                    outf.write(row)
            logger.info(f'Finish {chr}, total lines={cntLines:,}')
    print(f"DONE combine all chrs for {toolName}")



import sys

from string import Template

with open(sys.argv[1], 'r', encoding="utf-8") as file:
    instr = file.read()

readme_template = Template(instr)

replace_dict = {'dsname': sys.argv[2], 'outdir': sys.argv[3],
                'projectDir': sys.argv[4], 'workDir': sys.argv[5],
                'commandLine': sys.argv[6], 'runName': sys.argv[7],
                'start': sys.argv[8]
                }
readme_out = readme_template.substitute(replace_dict)
print(readme_out)



# -*- coding: utf-8 -*-
"""
Created on Thu Jul  9 18:03:39 2020

@author: akanksha
"""

import argparse
import warnings
from functools import reduce

import joblib
import pandas as pd
import sklearn
from pandas.core.common import SettingWithCopyWarning

warnings.simplefilter(action="ignore", category=SettingWithCopyWarning)

parser = argparse.ArgumentParser(description='Prediction from combined models for the reads.')

parser.add_argument('--methodsfile', '-i', type=str, required=True,
                    help='TSV file containing name and path of the method output tsv file. The output tsv file from the method should be in the format [ID,Pos,Strand,Score]. Can be compressed in gz.')

parser.add_argument('--model', '-m', choices=["default", "optimized"], required=True, type=str,
                    help='which model to select from default RF or optimized RF with max_depth 3 and n_estimator 10')

parser.add_argument('--output', '-o', type=str, required=True,
                    help='Where to store the outputs')

parser.add_argument('--model-base-dir', '-b', type=str, required=True,
                    help='Where is the model dir', default='/projects/li-lab/yang/test/METEORE')

options = parser.parse_args()


def mod_file(data_file_path):
    data_file = pd.read_csv(data_file_path, header=0, sep="\t")
    name = data_file_path.split("/")[-1].split(".")[0]  # HL60_DeepSignal-METEORE-perRead-score
    data_file.drop_duplicates(subset=['Chr', "ID", "Pos", "Strand"], inplace=True)  # add chr
    data_file.reset_index(inplace=True, drop=True)

    df_id_strand = data_file[["ID", "Strand"]]
    df_id_strand.drop_duplicates(subset=["ID", "Strand"], inplace=True)

    # mask = data_file.index[data_file.Strand == "-"].tolist() # select negative strand
    # data_file["Pos"][mask] = data_file["Pos"][mask] - 1 # merge to positive strand
    data_file.drop(["Strand"], axis=1, inplace=True)
    data_file.rename(columns={"Score": name}, inplace=True)
    data_file.reset_index(inplace=True, drop=True)

    # print(data_file)
    return (data_file, df_id_strand)


def main(mp, combine_file, combine_id_strand):
    loaded_model = joblib.load(open(mp, 'rb'))
    print(loaded_model)

    combine_file.dropna(inplace=True)
    X = combine_file[combine_file.columns[3:]]  # 2:
    X = sklearn.preprocessing.MinMaxScaler().fit_transform(X)
    prediction = pd.DataFrame(loaded_model.predict(X))  ##
    prediction_prob = pd.DataFrame(loaded_model.predict_proba(X))
    prediction.rename(columns={0: "Prediction"}, inplace=True)
    prediction_prob = prediction_prob[[1]]
    prediction_prob.rename(columns={1: "Prob_methylation"}, inplace=True)
    final_output = pd.concat([combine_file[combine_file.columns[:3]], prediction, prediction_prob], axis=1)  #:2

    final_output.dropna(inplace=True)
    import numpy as np
    final_output['Pos'] = final_output['Pos'].astype(np.int64)
    final_output['Prediction'] = final_output['Prediction'].astype(int)
    outdf = final_output.merge(combine_id_strand, how='left', on='ID')

    outdf.to_csv(options.output, header=True, index=None, sep='\t')
    print(f"save to {options.output}")


if __name__ == '__main__':
    # Used to find the randome forest model in Github
    METEORE_Dir = options.model_base_dir
    df_file = pd.read_csv(options.methodsfile, header=None, sep='\t')
    if options.model == "default":
        fillval = "default"
    else:
        fillval = "max_depth_3_n_estimator_10"
    modelname = '_'.join(df_file[0])
    mp = f'{METEORE_Dir}/saved_models/rf_model_' + fillval + '_' + modelname + '.model'
    dfs = []
    dfs_id_strand = []
    for i in df_file[1]:
        df, df_id_strand = mod_file(i)
        dfs.append(df)
        dfs_id_strand.append(df_id_strand)
    combine_file = reduce(lambda left, right: pd.merge(left, right, how='inner', on=["ID", "Chr", "Pos"]),
                          dfs)  # add chr
    combine_file.drop_duplicates(subset=["ID", "Chr", "Pos"], inplace=True)  # add chr
    combine_file.reset_index(inplace=True, drop=True)
    # print(combine_file)

    # outfn = options.output.replace('-perRead.combine.tsv.gz', '-intermediate.combinedf.tsv.gz')
    # combine_file.to_csv(outfn, sep='\t')
    #
    # print(f"save to {outfn}")

    # Used for add strand later
    combine_id_strand = pd.concat(dfs_id_strand)
    combine_id_strand.drop_duplicates(inplace=True)
    # print(combine_id_strand)
    if len(combine_file) > 0:
        main(mp, combine_file, combine_id_strand)  ##



"""
Sanity check for 5hmC detected by nanopore tools but not by BS-seq,
generate the agreement and discrepancy area between tool and BS-seq.

We will use these two areas to check the 5hmC level: we found that the discrepancy have a much higher 5hmC level.
"""
import gzip

import pandas as pd
from scipy.stats import pearsonr, wilcoxon

from nanocompare.eval_common import import_bgtruth, import_call, readLevelToSiteLevelWithCov, combineBGTruthList
from nanocompare.global_config import *


def output_cpg_set_0base(cpg_set, outfn, bgtruthDict=None):
    if bgtruthDict is None:
        raise Exception("Please specify bgtruthDict")
    outf = gzip.open(outfn, 'wt')
    for key in cpg_set:
        # key is (chr, start, strand)
        if isinstance(bgtruthDict[key], list) or isinstance(bgtruthDict[key], tuple):
            cov = bgtruthDict[key][1]
        else:
            cov = None
        if cov is None:
            outf.write(f"{key[0]}\t{key[1] - 1}\t{key[1]}\t.\t.\t{key[2]}\n")
        else:
            outf.write(f"{key[0]}\t{key[1] - 1}\t{key[1]}\t.\t.\t{key[2]}\t{cov}\n")
    outf.close()
    logger.info(f"save to {outfn}")


def correlation_calls(call1, call2, keySet):
    v1 = []
    v2 = []
    for key in keySet:
        if isinstance(call1[key], list) or isinstance(call1[key], tuple):
            vv1 = call1[key][0]
        else:
            vv1 = call1[key]

        if isinstance(call2[key], list) or isinstance(call2[key], tuple):
            vv2 = call2[key][0]
        else:
            vv2 = call2[key]
        v1.append(vv1)
        v2.append(vv2)
    coe, pvalue = pearsonr(v1, v2)
    return coe, pvalue


def wilcoxon_test_calls(call1, call2, keySet):
    v1 = []
    v2 = []
    for key in keySet:
        if isinstance(call1[key], list) or isinstance(call1[key], tuple):
            vv1 = call1[key][0]
        else:
            vv1 = call1[key]

        if isinstance(call2[key], list) or isinstance(call2[key], tuple):
            vv2 = call2[key][0]
        else:
            vv2 = call2[key]
        v1.append(vv1)
        v2.append(vv2)
    stats, pvalue = wilcoxon(v1, v2)
    return stats, pvalue
    pass


if __name__ == '__main__':
    set_log_debug_level()
    dsname = "APL"
    # dsname = "NA12878"

    logger.info(f"Processing {dsname}:")
    if dsname == "APL":
        mlmlFilename = "/pod/2/li-lab/Nanopore_compare/data/APL_5hmC_BSseq/APL.mlml.addstrand.bscov.oxbscov.selected.bed.gz"
        mlmlDict = import_bgtruth(mlmlFilename, encode="5hmc_ziwei", covCutoff=1, includeCov=True)
        logger.info(f"mlmlDict={len(mlmlDict):,}")

        bgtruthFilename = "/projects/li-lab/Nanopore_compare/data/APL/APL-bs_R1_val_1_bismark_bt2_pe.deduplicated.bismark.cov.convert.add.strand.tsv.gz"
        # bgtruthDict is key -> (meth_freq, cov)
        bgtruthDict = import_bgtruth(bgtruthFilename, encode="bismark", covCutoff=1, includeCov=True)
        logger.info(f"bgtruthDict={len(bgtruthDict):,}")

        newBgtruthDict = {}
        joinKeys = set(bgtruthDict.keys()).intersection(set(mlmlDict.keys()))
        for key in joinKeys:
            newBgtruthDict[key] = bgtruthDict[key]
        logger.info(f"newBgtruthDict={len(newBgtruthDict):,}")

        # megalodonDict: key-> (meth_freq, cov)
        megalodonFilename = "/pod/2/li-lab/Nanopore_compare/data/APL/APL.megalodon.per_read.sorted.bed.gz"
        megalodonDict = import_call(megalodonFilename, encode="Megalodon.ZW")
        megalodonDict = readLevelToSiteLevelWithCov(megalodonDict, minCov=3, toolname="Megalodon")

        # nanopolishDict: key-> (meth_freq, cov)
        nanopolishFilename = "/pod/2/li-lab/Nanopore_compare/data/APL/APL.nanopolish.methylation_calls.combine.tsv.gz"
        nanopolishDict = import_call(nanopolishFilename, encode="Nanopolish")
        nanopolishDict = readLevelToSiteLevelWithCov(nanopolishDict, minCov=3, toolname="Nanopolish")

        # DeepSignal
        deepsignalFilename = "/pod/2/li-lab/Nanopore_compare/data/APL/APL.deepsignal.call_mods.combine.tsv.gz"
        deepsignalDict = import_call(deepsignalFilename, encode="DeepSignal")
        deepsignalDict = readLevelToSiteLevelWithCov(deepsignalDict, minCov=3, toolname="DeepSignal")

        # Guppy
        guppyFilename = "/pod/2/li-lab/Nanopore_compare/data/APL/APL.Guppy.fast5mod.persite.raw.tsv.gz"
        guppyDict = import_call(guppyFilename, encode="Guppy")
        guppyDict = readLevelToSiteLevelWithCov(guppyDict, minCov=3, toolname="Guppy")

        logger.info(
            f"newBgtruthDict={len(newBgtruthDict)}, megalodonDict={len(megalodonDict)}, nanopolishDict={len(nanopolishDict)}, deepsignalDict={len(deepsignalDict)}, guppyDict={len(guppyDict)}")
    elif dsname == "NA12878":
        enable_cache = True
        using_cache = True

        bgtruth1Filename = "/projects/li-lab/Nanopore_compare/data/NA12878/ENCFF279HCL.bed.gz"
        # bgtruthDict is key -> (meth_freq, cov)
        bgtruth1Dict = import_bgtruth(bgtruth1Filename, encode="encode", covCutoff=1, includeCov=True,
                                      enable_cache=enable_cache, using_cache=using_cache)
        logger.info(f"bgtruth1Dict={len(bgtruth1Dict):,}")

        bgtruth2Filename = "/projects/li-lab/Nanopore_compare/data/NA12878/ENCFF835NTC.bed.gz"
        # bgtruthDict is key -> (meth_freq, cov)
        bgtruth2Dict = import_bgtruth(bgtruth2Filename, encode="encode", covCutoff=1, includeCov=True,
                                      enable_cache=enable_cache, using_cache=using_cache)
        logger.info(f"bgtruth2Dict={len(bgtruth2Dict):,}")

        bgtruthDict = combineBGTruthList([bgtruth1Dict, bgtruth2Dict], covCutoff=1)
        logger.info(f"combined bgtruthDict={len(bgtruthDict):,}")

        newBgtruthDict = bgtruthDict

        # newBgtruthDict = {}
        # joinKeys = set(bgtruthDict.keys()).intersection(set(mlmlDict.keys()))
        # for key in joinKeys:
        #     newBgtruthDict[key] = bgtruthDict[key]
        # logger.info(f"newBgtruthDict={len(newBgtruthDict):,}")

        # megalodonDict: key-> (meth_freq, cov)
        megalodonFilename = "/projects/li-lab/Nanopore_compare/data/NA12878/combine_allchrs/NA12878.megalodon.per_read.combine_allchrs.bed.gz"
        megalodonDict = import_call(megalodonFilename, encode="Megalodon", enable_cache=enable_cache,
                                    using_cache=using_cache)
        megalodonDict = readLevelToSiteLevelWithCov(megalodonDict, minCov=3, toolname="Megalodon")

        # nanopolishDict: key-> (meth_freq, cov)
        nanopolishFilename = "/projects/li-lab/Nanopore_compare/data/NA12878/combine_allchrs/NA12878.nanopolish.methylation_calls.combine_allchrs.tsv.gz"
        nanopolishDict = import_call(nanopolishFilename, encode="Nanopolish", enable_cache=enable_cache,
                                     using_cache=using_cache)
        nanopolishDict = readLevelToSiteLevelWithCov(nanopolishDict, minCov=3, toolname="Nanopolish")

        # DeepSignal
        deepsignalFilename = "/projects/li-lab/Nanopore_compare/data/NA12878/combine_allchrs/NA12878.deepsignal.call_mods.combine_allchrs.tsv.gz"
        deepsignalDict = import_call(deepsignalFilename, encode="DeepSignal", enable_cache=enable_cache,
                                     using_cache=using_cache)
        deepsignalDict = readLevelToSiteLevelWithCov(deepsignalDict, minCov=3, toolname="DeepSignal")

        # Guppy
        guppyFilename = "/projects/li-lab/Nanopore_compare/data/NA12878/combine_allchrs/NA12878.guppy.fast5mod_site_level.combine_allchrs.tsv.gz"
        guppyDict = import_call(guppyFilename, encode="Guppy", enable_cache=enable_cache,
                                using_cache=using_cache)
        guppyDict = readLevelToSiteLevelWithCov(guppyDict, minCov=3, toolname="Guppy")

        logger.info(
            f"megalodonDict={len(megalodonDict)}, nanopolishDict={len(nanopolishDict)}, deepsignalDict={len(deepsignalDict)}, guppyDict={len(guppyDict)}")

    for callDict, callName in zip([megalodonDict, nanopolishDict, deepsignalDict, guppyDict],
                                  ['Megalodon', 'Nanopolish', 'DeepSignal', 'Guppy']):
        logger.debug(f"Analyze tool={callName}, CpGs={len(callDict):,}")
        joinSet = set(newBgtruthDict.keys()).intersection(set(callDict.keys()))

        coe, pvalue = correlation_calls(newBgtruthDict, callDict, joinSet)
        logger.info(f"BS-seq vs. {callName} COE={coe:.3f}, pvalue={pvalue:e}, CpGs={len(joinSet):,}")

        ret_cons_let_05 = set()
        ret_cons_let_10 = set()
        ret_diff_get_20 = set()
        ret_diff_get_40 = set()

        agre_let_05_5mc_bgtruth_list = []
        agre_let_05_5mc_tool_list = []
        agre_let_10_5mc_bgtruth_list = []
        agre_let_10_5mc_tool_list = []

        diff_get_20_5mc_bgtruth_list = []
        diff_get_20_5mc_tool_list = []
        diff_get_40_5mc_bgtruth_list = []
        diff_get_40_5mc_tool_list = []

        for key in joinSet:
            if isinstance(newBgtruthDict[key], list) or isinstance(newBgtruthDict[key], tuple):
                bg_methfreq = newBgtruthDict[key][0]
            else:
                bg_methfreq = newBgtruthDict[key]
            tool_methfreq = callDict[key][0]
            if abs(tool_methfreq - bg_methfreq) < 0.05:
                ret_cons_let_05.add(key)
                agre_let_05_5mc_bgtruth_list.append(bg_methfreq)
                agre_let_05_5mc_tool_list.append(tool_methfreq)

            if abs(tool_methfreq - bg_methfreq) < 0.10:
                ret_cons_let_10.add(key)
                agre_let_10_5mc_bgtruth_list.append(bg_methfreq)
                agre_let_10_5mc_tool_list.append(tool_methfreq)

            if abs(tool_methfreq - bg_methfreq) > 0.2:
                ret_diff_get_20.add(key)
                diff_get_20_5mc_bgtruth_list.append(bg_methfreq)
                diff_get_20_5mc_tool_list.append(tool_methfreq)

            if abs(tool_methfreq - bg_methfreq) > 0.4:
                ret_diff_get_40.add(key)
                diff_get_40_5mc_bgtruth_list.append(bg_methfreq)
                diff_get_40_5mc_tool_list.append(tool_methfreq)

        ## Output for scatter plotting
        df = pd.DataFrame({"BS-seq": agre_let_10_5mc_bgtruth_list, callName: agre_let_10_5mc_tool_list})
        outfn = os.path.join(pic_base_dir, f"{dsname}.outcome2.bsseq.{callName}.let10.agreement.csv.gz")
        df.to_csv(outfn, index=False)

        df = pd.DataFrame({"BS-seq": agre_let_05_5mc_bgtruth_list, callName: agre_let_05_5mc_tool_list})
        outfn = os.path.join(pic_base_dir, f"{dsname}.outcome2.bsseq.{callName}.let05.agreement.csv.gz")
        df.to_csv(outfn, index=False)

        df = pd.DataFrame({"BS-seq": diff_get_20_5mc_bgtruth_list, callName: diff_get_20_5mc_tool_list})
        outfn = os.path.join(pic_base_dir, f"{dsname}.outcome2.bsseq.{callName}.get20.discrepency.csv.gz")
        df.to_csv(outfn, index=False)

        df = pd.DataFrame({"BS-seq": diff_get_40_5mc_bgtruth_list, callName: diff_get_40_5mc_tool_list})
        outfn = os.path.join(pic_base_dir, f"{dsname}.outcome2.bsseq.{callName}.get40.discrepency.csv.gz")
        df.to_csv(outfn, index=False)

        logger.info(
            f"tool={callName}:  ret_cons_let_05={len(ret_cons_let_05)}, ret_cons_let_10={len(ret_cons_let_10)}, ret_diff_get_20={len(ret_diff_get_20)}, ret_diff_get_40={len(ret_diff_get_40)}")

        # stats_let_05, pv_let05 = wilcoxon_test_calls(newBgtruthDict, callDict, ret_cons_let_05)
        # stats_let_10, pv_let10 = wilcoxon_test_calls(newBgtruthDict, callDict, ret_cons_let_10)
        # stats_get_20, pv_get20 = wilcoxon_test_calls(newBgtruthDict, callDict, ret_diff_get_20)
        # stats_get_40, pv_get40 = wilcoxon_test_calls(newBgtruthDict, callDict, ret_diff_get_40)
        #
        # logger.info(
        #     f"tool={callName} wilcoxon test:\n <5% = ({stats_let_05}, {pv_let05});\n <10% = ({stats_let_10},{pv_let10});\n >20% = ({stats_get_20}, {pv_get20});\n >40% = ({stats_get_40}, {pv_get40})\n")

        ## Output for DeepTool plotting 5hmC
        outfn = f"{dsname}.bgtruth.{callName}.regions.cons.let.05.bed.gz"
        output_cpg_set_0base(ret_cons_let_05, os.path.join(pic_base_dir, outfn), bgtruthDict=bgtruthDict)

        outfn = f"{dsname}.bgtruth.{callName}.regions.cons.let.10.bed.gz"
        output_cpg_set_0base(ret_cons_let_10, os.path.join(pic_base_dir, outfn), bgtruthDict=bgtruthDict)

        outfn = f"{dsname}.bgtruth.{callName}.regions.diff.get.20.bed.gz"
        output_cpg_set_0base(ret_diff_get_20, os.path.join(pic_base_dir, outfn), bgtruthDict=bgtruthDict)

        outfn = f"{dsname}.bgtruth.{callName}.regions.diff.get.40.bed.gz"
        output_cpg_set_0base(ret_diff_get_40, os.path.join(pic_base_dir, outfn), bgtruthDict=bgtruthDict)

    logger.info(f"sanity_check 5hmc DONE for {dsname}: generate the agreement and discrepancy regions.")



#!/usr/bin/env python3
# @Author   : Yang Liu
# @FileName : site_level_eval.py
# @Software : NANOME project
# @Organization : JAX Li Lab
# @Website  : https://github.com/TheJacksonLaboratory/nanome

"""
Generate site-level methylation correlation results in nanome paper.
"""

import argparse

import pybedtools

from nanocompare.eval_common import *
from nanocompare.global_settings import get_tool_name, Top3ToolNameList, ToolNameList, save_done_file, \
    narrowCoordNameList, cg_density_coord_name_list, rep_coord_name_list, nanome_version


def get_nsites_in_regions(callSet, bedfn, tagname):
    subset = filter_cpgkeys_using_bedfile(callSet, bedfn)
    ret = {tagname: len(subset)}
    return ret


def summary_cpgs_stats_results_table():
    """
    Study and summary each tool joined with bg-truth results, make table as dataframe
    :return:
    """
    logger.debug(f"Report number of sites by methylation calling tools in each region, take times...")
    dataset = []
    bgtruthCpGs = set(list(bgTruth.keys()))
    joinedSet = None
    unionSet = set()

    retList = []
    logger.debug("Start to study coverage now:")
    for toolname in loaded_callname_list:
        ## CpG sites set with cov >= cutoff(3)
        logger.info(f'Study CPG coverage for tool={toolname}')
        logger.info(f"Memory report: {get_current_memory_usage()}")
        callSet = list(set(callresult_dict_cov3[toolname].keys()))

        if not joinedSet:
            joinedSet = set(callSet)
        else:
            joinedSet = joinedSet.intersection(callSet)
        unionSet = unionSet.union(callSet)
        toolOverlapBGTruthCpGs = bgtruthCpGs.intersection(set(list(callresult_dict_cov3[toolname].keys())))
        row_dict = {f'CpG sites in BG-Truth cov>={bgtruthCutt}': len(bgtruthCpGs),
                    'Total CpG sites by Nanopore tool': call_cov1_cpg_sites[toolname],
                    f'Total CpG sites by tool cov>={minToolCovCutt}': len(callresult_dict_cov3[toolname]),
                    'Joined CpG sites with BG-Truth': len(toolOverlapBGTruthCpGs)}
        row_dict.update({'Total calls by Nanopore reads': call_cov1_calls[toolname]})

        callBed = calldict2bed(callSet)

        # Add coverage of every regions by each tool here
        for (bedfn, tagname, region_bed) in tqdm(
                region_bed_list):  # calculate how overlap with Singletons, Non-Singletons, etc.
            if not args.large_mem:  # load in demand
                region_bed = get_region_bed_tuple(bedfn, enable_base_detection_bedfile=not args.disable_bed_check)[2]

            if region_bed is None:
                logger.warning(f"region name={tagname} is not found")
                continue
            intersect_bed = intersect_bed_regions(callBed, region_bed, bedfn)
            ret = {tagname: len(intersect_bed)}
            retList.append(ret)
        if concordant_bed is not None:
            intersect_bed = intersect_bed_regions(callBed, concordant_bed)
            ret = {'Concordant': len(intersect_bed)}
            retList.append(ret)

        if discordant_bed is not None:
            intersect_bed = intersect_bed_regions(callBed, discordant_bed)
            ret = {'Discordant': len(intersect_bed)}
            retList.append(ret)

        retDict = {}
        for e in retList:
            retDict.update(e)
        logger.debug(f"retDict={retDict}")

        ## Sanity check
        sum_sing_nonsingle = retDict['Singletons'] + retDict['Non-singletons']
        if 'CG_20' in retDict and 'CG_40' in retDict and 'CG_60' in retDict and 'CG_80' in retDict and 'CG_100' in retDict:
            sum_cg = retDict['CG_20'] + retDict['CG_40'] + retDict['CG_60'] + retDict['CG_80'] + retDict['CG_100']
        else:
            sum_cg = 0
        total_sites = row_dict[f'Total CpG sites by tool cov>={minToolCovCutt}']
        logger.debug(
            f"\n\nSanity check: sum_sing_nonsingle={sum_sing_nonsingle:,}; sum_cg={sum_cg:,}; total={total_sites:,}")

        if sum_sing_nonsingle != total_sites:
            logger.error(
                f"Sanity check for {toolname}, total_sites={total_sites:,}, sum_sing_nonsingle={sum_sing_nonsingle:,}, some non-singletons are not captered by bed file")
            retDict['Non-singletons'] = total_sites - retDict['Singletons']
            logger.error(f"Updated, retDict={retDict}")
        row_dict.update(retDict)
        dataset.append(row_dict)
        logger.debug(f'BG-Truth join with {toolname} get {len(toolOverlapBGTruthCpGs):,} CpGs')
        # outfn = os.path.join(out_dir, f'{RunPrefix}-joined-cpgs-bgtruth-{name1}-bsCov{bgtruthCutt}-minCov{minToolCovCutt}-baseCount{baseFormat}.bed')
        # save_keys_to_bed(overlapCpGs, outfn)

    # add additional rows for Joined count
    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(joinedSet)}
    dataset.append(new_row_dict)

    # add additional row for Unioned count
    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(unionSet)}
    dataset.append(new_row_dict)

    # also report top3 joined and union set
    top3JointSet = None
    top3UnionSet = set()

    for callname in Top3ToolNameList:
        toolSet = top3_cpg_set_dict[callname]
        if not top3JointSet:
            top3JointSet = toolSet
        else:
            top3JointSet = top3JointSet.intersection(toolSet)
        top3UnionSet = top3UnionSet.union(toolSet)

    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top3JointSet)}
    dataset.append(new_row_dict)

    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top3UnionSet)}
    dataset.append(new_row_dict)

    # also report top4 joined and union set
    top4JointSet = None
    top4UnionSet = set()

    for callname in ToolNameList[:4]:
        if callname not in callresult_dict_cov3:
            continue
        toolSet = set(callresult_dict_cov3[callname].keys())
        if not top4JointSet:
            top4JointSet = toolSet
        else:
            top4JointSet = top4JointSet.intersection(toolSet)
        top4UnionSet = top4UnionSet.union(toolSet)

    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top4JointSet)}
    dataset.append(new_row_dict)

    new_row_dict = {f'Total CpG sites by tool cov>={minToolCovCutt}': len(top4UnionSet)}
    dataset.append(new_row_dict)

    df = pd.DataFrame(dataset, index=loaded_callname_list +
                                     ['Joined', 'Union', 'TOP3 Joined', 'TOP3 Union', 'TOP4 Joined', 'TOP4 Union'])

    logger.debug(df)

    outfn = os.path.join(out_dir,
                         f'{RunPrefix}-summary-bgtruth-tools-bsCov{bgtruthCutt}-minCov{minToolCovCutt}.table.s10.xlsx')
    df.to_excel(outfn)
    logger.debug(f'save to {outfn}\n')


def save_meth_corr_data(callresult_dict, bgTruth, reportCpGSet, outfn):
    """
    Save meth freq and cov results into csv file
    :param callresult_dict:
    :param bgTruth:
    :param reportCpGSet:
    :param outfn:
    :return:
    """
    # outfile = open(outfn, 'w')
    outfile = gzip.open(outfn, 'wt')

    header_list = ['chr', 'start', 'end', 'BGTruth_freq', 'BGTruth_cov', 'strand']

    # Output header
    for name in loaded_callname_list:
        header_list.extend([f'{name}_freq', f'{name}_cov'])
    outfile.write(sep.join(header_list))
    outfile.write("\n")

    for cpg in reportCpGSet:
        if baseFormat == 0:
            end = cpg[1] + 1
        elif baseFormat == 1:
            end = cpg[1]

        # Ouput ground truth
        row_list = [cpg[0], str(cpg[1]), str(end), f'{bgTruth[cpg][0]:.3f}', str(bgTruth[cpg][1]), cpg[2]]

        # Output each tool results
        for name in loaded_callname_list:
            if cpg in callresult_dict[name]:  # if cpg is in tool results
                row_list.extend([f'{callresult_dict[name][cpg][0]:.3f}', f'{callresult_dict[name][cpg][1]}'])
            else:  # if cpg key is not exist, we use NA as ''
                row_list.extend(['', ''])
        outfile.write(sep.join(row_list))
        outfile.write("\n")
    outfile.close()
    logger.debug(f"save to {outfn}\n")


def parse_arguments():
    """
    :return:
    """
    parser = argparse.ArgumentParser(prog='site_level_eval (NANOME)',
                                     description='Site-level correlation analysis in nanome paper')
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s v{nanome_version}')
    parser.add_argument('--dsname', type=str, help="dataset name", required=True)
    parser.add_argument('--runid', type=str, help="running prefix", required=True)
    parser.add_argument('--calls', nargs='+', help='all ONT call results <tool-name>:<file-name> seperated by spaces',
                        required=True)
    parser.add_argument('--bgtruth', type=str, help="background truth file <encode-type>:<file-name1>;<file-name1>",
                        required=True)
    parser.add_argument('--beddir', type=str, help="base dir for concordant/discordant bed files",
                        default=None)
    parser.add_argument('--min-bgtruth-cov', type=int, help="cutoff for coverage in bg-truth", default=5)
    parser.add_argument('--toolcov-cutoff', type=int, help="cutoff for coverage in nanopore tools", default=3)
    parser.add_argument('--sep', type=str, help="seperator for output csv file", default=',')
    parser.add_argument('--processors', type=int, help="running processors", default=1)
    parser.add_argument('-o', type=str, help="output dir", default=pic_base_dir)
    parser.add_argument('--gen-venn', help="generate CpGs for venn data analysis", action='store_true')
    parser.add_argument('--summary-coverage', help="generate summary table for coverage at each region",
                        action='store_true')
    parser.add_argument('--region-coe-report', help="report table for PCC value at each region",
                        action='store_true')
    parser.add_argument('--enable-cache', help="if enable cache functions", action='store_true')
    parser.add_argument('--using-cache', help="if use cache files", action='store_true')
    parser.add_argument('--plot', help="plot the correlation matrix figure", action='store_true')
    parser.add_argument('--bedtools-tmp', type=str, help='bedtools temp dir', default=temp_dir)
    parser.add_argument('--genome-annotation', type=str, help='genome annotation dir',
                        default=os.path.join(data_base_dir, 'genome-annotation'))
    parser.add_argument('--large-mem', help="if using large memory (>100GB)", action='store_true')
    parser.add_argument('--disable-bed-check', help="if disable checking the 0/1 base format for genome annotations",
                        action='store_true')
    parser.add_argument('--debug', help="if output debug info", action='store_true')
    return parser.parse_args()


if __name__ == '__main__':
    args = parse_arguments()

    if args.debug:
        set_log_debug_level()
    else:
        set_log_info_level()

    ## Set tmp dir for bedtools
    os.makedirs(args.bedtools_tmp, exist_ok=True)
    pybedtools.helpers.set_tempdir(args.bedtools_tmp)

    # cache function same with read level
    enable_cache = args.enable_cache
    using_cache = args.using_cache

    # runid is always like 'MethCorr-K562_WGBS_2Reps', remove first word as RunPrefix like K562_WGBS_2Reps
    RunPrefix = args.runid.replace('MethCorr-', '')

    # tool coverage cutoff 1, or 3, 5
    minToolCovCutt = args.toolcov_cutoff

    # bgtruth coverage cutoff 1, or 5, 10  --min-bgtruth-cov
    bgtruthCutt = args.min_bgtruth_cov

    # We import and report use 1-base start format
    baseFormat = 1

    # output csv seperator: , or tab
    sep = args.sep

    out_dir = os.path.join(args.o, args.runid)
    os.makedirs(out_dir, exist_ok=True)
    logger.info(f'Output to dir:{out_dir}')

    # Add logging files also to result output dir
    add_logging_file(os.path.join(out_dir, 'run-results.log'))

    logger.debug(args)
    logger.debug(f'\n\n####################\n\n')

    # we import multiple (1 or 2) replicates and join them
    encode, fnlist = args.bgtruth.split(':')
    fnlist = fnlist.split(';')

    if len(fnlist) > 2:
        raise Exception(f'Currently only support the number of bgtruth upto two, but found more: {fnlist}')

    logger.debug(f'BGTruth fnlist={fnlist}, encode={encode}')

    bgTruthList = []
    for fn in fnlist:
        if len(fn) == 0:  # incase of input like 'bismark:/a/b/c;'
            continue
        # import if cov >= 1 firstly, then after join two replicates step, remove low coverage
        bgTruth1 = import_bgtruth(fn, encode, covCutoff=1, baseFormat=baseFormat, includeCov=True,
                                  using_cache=using_cache, enable_cache=enable_cache)
        bgTruthList.append(bgTruth1)

    # Combine one/two replicates, using cutoff=1 or 5
    bgTruth = combineBGTruthList(bgTruthList, covCutoff=bgtruthCutt)

    logger.info(f'Combined BS-seq data (cov>={bgtruthCutt}), all methylation level sites={len(bgTruth):,}')

    logger.debug(f'\n\n####################\n\n')

    callfn_dict = defaultdict()  # callname -> filename

    # callname -> [call0, call1], call0 is no-filter results, call1 is filter by cutoff, and convert to [meth-freq, meth-cov] results.
    callresult_dict_cov1 = defaultdict()
    call_cov1_cpg_sites = defaultdict(int)
    call_cov1_calls = defaultdict(int)

    callresult_dict_cov3 = defaultdict()
    loaded_callname_list = []

    for callstr in args.calls:
        callencode, callfn = callstr.split(':')

        if len(callfn) == 0:
            continue

        callname = get_tool_name(callencode)
        callfn_dict[callname] = callfn

        # We do now allow import DeepMod.C for site level evaluation, in current version
        if callencode == 'DeepMod.C':
            raise Exception(
                f'{callencode} is not allowed for site level evaluation, please use DeepMod.Cluster file here')

        loaded_callname_list.append(callname)

        # For site level evaluation, only need (freq, cov) results, no score needed. Especially for DeepMod, we must import as freq and cov format from DeepMod.Cluster encode
        # Do not filter bgtruth, because we use later for overlapping (without bg-truth)
        callresult_dict_cov1[callname] = import_call(callfn, callencode, baseFormat=baseFormat,
                                                     enable_cache=enable_cache, using_cache=using_cache,
                                                     include_score=False, siteLevel=True)

        # Stats the total cpgs and calls for each calls
        cnt_calls = 0
        for cpg in callresult_dict_cov1[callname]:
            cnt_calls += len(callresult_dict_cov1[callname][cpg])
        call_cov1_calls[callname] = cnt_calls
        call_cov1_cpg_sites[callname] = len(callresult_dict_cov1[callname])

    logger.info(f"Import calls from tools done for toollist={list(call_cov1_cpg_sites.keys())}")
    logger.info(f"Memory report: {get_current_memory_usage()}")

    logger.debug(f'Start apply cutoff={minToolCovCutt} to methylation calls, take time')
    # Cutoff of read cov >= 1 or 3, 5 for nanopore tools
    for callname in loaded_callname_list:
        callresult_dict_cov3[callname] = readLevelToSiteLevelWithCov(callresult_dict_cov1[callname],
                                                                     minCov=minToolCovCutt, toolname=callname)
        ## Destroy cov1 for memory saving
        del callresult_dict_cov1[callname]

    logger.debug(f'\n\n####################\n\n')

    top3_cpg_set_dict = defaultdict()
    for callname in Top3ToolNameList:
        top3_cpg_set_dict[callname] = set(callresult_dict_cov3[callname].keys())

    if args.gen_venn:
        logger.info('CPG overlapping analysis')
        logger.debug(
            f"Start gen venn data for each tool (cov>={minToolCovCutt}) and BS-seq (cov>={args.min_bgtruth_cov})")

        # Generate all tools and bsseq covered cpgs files for set evaluation
        logger.debug("We generate sets file for each tool and bg-truth")
        venn_outdir = os.path.join(out_dir, 'venn_data')
        os.makedirs(venn_outdir, exist_ok=True)

        bg_cpgs = bgTruth.keys()
        outfn = os.path.join(venn_outdir, f'{args.dsname}.bgtruth.cpg.sites.cov{args.min_bgtruth_cov}.setsfile.txt.gz')
        ontcalls_to_setsfile_for_venn_analysis(bg_cpgs, outfn)
        for callname in ToolNameList:
            if callname not in callresult_dict_cov3:
                continue
            call_keys = callresult_dict_cov3[callname].keys()
            outfn = os.path.join(venn_outdir,
                                 f'{args.dsname}.{callname}.cpg.sites.cov{args.toolcov_cutoff}.setsfile.txt.gz')
            ontcalls_to_setsfile_for_venn_analysis(call_keys, outfn)
        logger.debug(f'\n\n####################\n\n')

    logger.debug(f"Start getting intersection (all joined) sites by tools and bgtruth")
    coveredCpGs = set(list(bgTruth.keys()))
    coveredCpGs001 = set(list(bgTruth.keys()))

    sitesDataset = defaultdict(list)

    for name in loaded_callname_list:
        coveredCpGs = coveredCpGs.intersection(set(list(callresult_dict_cov3[name].keys())))
        logger.debug(f'Join {name} get {len(coveredCpGs):,} CpGs')

        joinBSWithEachToolSet = coveredCpGs001.intersection(set(list(callresult_dict_cov3[name].keys())))
        sitesDataset['Dataset'].append(args.dsname)
        sitesDataset['Method'].append(name)
        sitesDataset[f'Sites-cov{args.toolcov_cutoff}'].append(len(callresult_dict_cov3[name]))
        sitesDataset[f'BS-seq-cov{args.min_bgtruth_cov}-all'].append(len(coveredCpGs001))
        sitesDataset[f'Join-with-BSseq-cov{args.min_bgtruth_cov}-all'].append(len(joinBSWithEachToolSet))
    df = pd.DataFrame.from_dict(sitesDataset)
    outfn = os.path.join(out_dir,
                         f'{args.dsname}.tools.cov{args.toolcov_cutoff}.join.with.bsseq.cov{args.min_bgtruth_cov}.site.level.report.csv')
    df.to_csv(outfn)

    # Output sites report for all tools and BS-seq
    logger.info(f"Joined {len(coveredCpGs):,} CpGs are covered by all tools (cov >= {args.toolcov_cutoff}) and BS-seq (cov >= {args.min_bgtruth_cov})")
    logger.debug('Output data of meth-freq and coverage on joined CpG sites as datasets for correlation analysis')
    outfn_joined = os.path.join(out_dir,
                                f"Meth_corr_plot_data_joined-{RunPrefix}-bsCov{bgtruthCutt}-minToolCov{minToolCovCutt}-baseFormat{baseFormat}.csv.gz")
    save_meth_corr_data(callresult_dict_cov3, bgTruth, coveredCpGs, outfn_joined)
    outfn_joined_sorted = os.path.join(out_dir,
                                       f"Meth_corr_plot_data_joined-{RunPrefix}-bsCov{bgtruthCutt}-minToolCov{minToolCovCutt}-baseFormat{baseFormat}.sorted.csv.gz")
    sort_bed_file(infn=outfn_joined, outfn=outfn_joined_sorted, has_header=True)
    os.remove(outfn_joined)

    logger.debug(
        'Output data of meth-freq and coverage on bgTruth related CpG sites as datasets for correlation analysis')
    outfn_bgtruth = os.path.join(out_dir,
                                 f"Meth_corr_plot_data_bgtruth-{RunPrefix}-bsCov{bgtruthCutt}-minToolCov{minToolCovCutt}-baseFormat{baseFormat}.csv.gz")
    save_meth_corr_data(callresult_dict_cov3, bgTruth, set(list(bgTruth.keys())), outfn_bgtruth)
    outfn_bgtruth_sroted = os.path.join(out_dir,
                                        f"Meth_corr_plot_data_bgtruth-{RunPrefix}-bsCov{bgtruthCutt}-minToolCov{minToolCovCutt}-baseFormat{baseFormat}.sorted.csv.gz")
    sort_bed_file(infn=outfn_bgtruth, outfn=outfn_bgtruth_sroted, has_header=True)
    os.remove(outfn_bgtruth)

    # Report correlation matrix here
    df = pd.read_csv(outfn_joined_sorted, sep=sep)
    df = df.filter(regex='_freq$', axis=1)
    cordf = df.corr()
    logger.debug(f'Correlation matrix is:\n{cordf}')
    corr_outfn = os.path.join(out_dir,
                              f'Meth_corr_plot_data-{RunPrefix}-correlation-matrix-toolcov{minToolCovCutt}-bsseqcov{bgtruthCutt}.xlsx')
    cordf.to_excel(corr_outfn)

    logger.debug(f'\n\n####################\n\n')

    if args.region_coe_report or args.summary_coverage:
        ## load region bed list
        logger.debug("Create region bed list firstly, take times......")

        # Evaluated all region filename lists, bed objects
        # assume all files are located in args.genome_annotation dir
        regions_full_filepath = [os.path.join(args.genome_annotation, cofn) for cofn in narrowCoordNameList[1:]] + \
                                [os.path.join(args.genome_annotation, cofn) for cofn in cg_density_coord_name_list] + \
                                [os.path.join(args.genome_annotation, cofn) for cofn in rep_coord_name_list]
        # logger.debug(f"Evaluated regions: {regions_full_filepath}")

        if args.large_mem:  # load all in memory
            region_bed_list = get_region_bed_pairs_list_mp(regions_full_filepath, processors=args.processors, enable_base_detection_bedfile=not args.disable_bed_check)
        else:  # load bed coord later
            region_bed_list = [(infn, map_region_fn_to_name(infn), None,)
                               for infn in regions_full_filepath]

        if args.beddir:  # add concordant and discordant region coverage if needed
            logger.debug(f'We use Concordant and Discordant BED file at basedir={args.beddir}')
            concordantFileName = find_bed_filename(basedir=args.beddir,
                                                   pattern=f'{args.dsname}*hg38_nonsingletons*.concordant.bed.gz')
            concordant_bed = get_region_bed(concordantFileName)
            discordantFileName = find_bed_filename(basedir=args.beddir,
                                                   pattern=f'{args.dsname}*hg38_nonsingletons*.discordant.bed.gz')
            discordant_bed = get_region_bed(discordantFileName)
        else:
            concordant_bed = None
            discordant_bed = None

    if args.region_coe_report:
        eval_genomic_context_tuple = [('x.x.Genome-wide', 'Genome-wide', None)] + region_bed_list
        if concordant_bed is not None:
            eval_genomic_context_tuple += [(concordantFileName, 'Concordant', concordant_bed,)]
        if discordant_bed is not None:
            eval_genomic_context_tuple += [(discordantFileName, 'Discordant', discordant_bed,)]

        logger.debug(f"Evaluated on regions: {eval_genomic_context_tuple}")

        # file like: Meth_corr_plot_data_joined-TestData_RRBS_2Reps-bsCov1-minToolCov1-baseFormat1.sorted.csv.gz
        fnlist = glob.glob(os.path.join(out_dir, f'Meth_corr_plot_data_joined-*bsCov{args.min_bgtruth_cov}-minToolCov{args.toolcov_cutoff}*.csv.gz'))
        if len(fnlist) < 1:
            raise Exception(f'Found no file for fnlist={fnlist}, for dir={out_dir}')
        logger.debug(f'Find file: {fnlist}')

        basefn = os.path.basename(fnlist[0])
        tagname = basefn.replace('Meth_corr_plot_data_joined-', '')
        dsname = tagname[:tagname.find('_')]

        logger.info(f"Start report PCC in difference genomic regions based on file={fnlist[0]}, dsname={dsname}")
        correlation_report_on_regions(fnlist[0], bed_tuple_list=eval_genomic_context_tuple, dsname=dsname,
                                      outdir=out_dir, large_mem=args.large_mem,
                                      enable_base_detection_bedfile=not args.disable_bed_check)

    if args.summary_coverage:
        logger.info("Start summarize CPG coverage at each regions")
        summary_cpgs_stats_results_table()
        logger.info(f"Memory report: {get_current_memory_usage()}")

    if args.plot:
        # plot fig5a of correlation plot
        command = f"set -x; plot_figure.py fig5a -i {outfn_joined_sorted} -o {out_dir}"
        subprocess.Popen(command, shell=True, stdout=subprocess.PIPE).stdout.read().decode("utf-8")

    save_done_file(out_dir)
    logger.info(f"Memory report: {get_current_memory_usage()}")
    logger.info("### Site level correlation analysis DONE")



import glob
import os
from collections import defaultdict

import pandas as pd

from nanocompare.global_config import pic_base_dir

baseDir = '/projects/li-lab/Nanopore_compare/suppdata/basecalls.qc'

flist = glob.glob(os.path.join(baseDir, '*basecall.sequencing_summary.txt'))

cutoff = 10000
print(f"We define long reads with length > {cutoff:,}")

dataset = defaultdict(list)

for fn in flist:
    basename = os.path.basename(fn)
    dsname = basename[0:basename.find('-')]
    df = pd.read_csv(fn, sep='\t')
    total_longreads = sum(df['sequence_length_template'] > cutoff)
    portion_longreads = total_longreads / len(df)
    print(
        f"Dataset={dsname}, total_reads={len(df):,}, total_longreads={total_longreads:,}, portion_longreads={portion_longreads * 100:.2f}%")
    dataset['Dataset'].append(dsname)
    dataset['total_reads'].append(len(df))
    dataset['total_longreads'].append(total_longreads)
    dataset['portion_longreads'].append(portion_longreads)

outdf = pd.DataFrame.from_dict(dataset)
outfn = os.path.join(pic_base_dir, f'datasets_long_reads_cutoff_{cutoff}_table.csv')
outdf.to_csv(outfn)
print(f"save to {outfn}")

"""
dsname=NA19240, total_reads=6,219,931, total_longreads=2,286,093, portion_longreads=36.75%
dsname=NA12878, total_reads=14,791,249, total_longreads=4,776,148, portion_longreads=32.29%
dsname=HL60, total_reads=189,105, total_longreads=63,665, portion_longreads=33.67%
dsname=APL, total_reads=1,154,349, total_longreads=204,778, portion_longreads=17.74%
dsname=K562, total_reads=263,019, total_longreads=82,727, portion_longreads=31.45%
"""
